#!/usr/bin/perl
# exportProfiles.pl script 
# get SNMP / IPDomain information and export DiscoveryProfiles from DA
#
# Prerequisites: tested with perl 5.18
#	typically, REST::Client and XML::Simple modules must be installed (cpan)
#	v1.1	March 30, 2015
#			supports a mix of (multiple) snmpv1 and snmpv3 profiles

# perl exportProfiles.pl IMDA:8581 discPrfExp.xml profileAssociations.csv
#
# DISCLAIMER
# This script is provided AS IS and no warranties are provided or made.  
# For more information refer to http://www.ca.com/us/legal/terms-for-user-sites.aspx
#
use strict;
use REST::Client;
use XML::LibXML;

my $num_args = $#ARGV + 1;
if ($num_args != 3) {
  print "\nUsage: getProfiles.pl DA_host_and_port (e.g. IMDA:8581) export_file association_file \n";
  exit;
}
my $input=$ARGV[0];
my $outfile=$ARGV[1];
my $assocfilename=$ARGV[2];

open(EXPORT,">".$outfile)
  or die "error when opening $outfile for output $!\n";
open(ASSOC,">".$assocfilename)
  or die "error when opening $outfile for output $!\n";

my @snmptable;									# SNMPv1/v2c/v3 profile table (DA id, name)
my @domaintable;								# ip domain table (DA id, name)

my $host = "http://$input";
my $client = REST::Client->new();
$client->setHost($host);
$client->addHeader('Accept', 'application/xml');

### process DA snmp profiles
##################################################################################################

$client->GET('/rest/profiles/snmpv1/');
# print $client->responseContent();

my $parser = XML::LibXML->new();
my $snmpprofilelist    = $parser->load_xml(string => $client->responseContent());

foreach my $snmpprof ($snmpprofilelist->findnodes('/SNMPv1ProfileList/SNMPv1Profile')) {
    my($title) = $snmpprof->findnodes('./CommunicationProfile/ProfileName');
    # print $title->to_literal, "\n";
	my @commrow;
	push @commrow, $title->to_literal;
    my($title) = $snmpprof->findnodes('./ID');
    # print $title->to_literal, "\n";
	push @commrow, $title->to_literal;
	push @commrow, "SNMPv1";
	push @snmptable, \@commrow;
}

$client->GET('/rest/profiles/snmpv3/');
# print $client->responseContent();

my $snmpprofilelist    = $parser->load_xml(string => $client->responseContent());

foreach my $snmpprof ($snmpprofilelist->findnodes('/SNMPv3ProfileList/SNMPv3Profile')) {
    my($title) = $snmpprof->findnodes('./CommunicationProfile/ProfileName');
    # print $title->to_literal, "\n";
	my @commrow;
	push @commrow, $title->to_literal;
    my($title) = $snmpprof->findnodes('./ID');
    # print $title->to_literal, "\n";
	push @commrow, $title->to_literal;
	push @commrow, "SNMPv3";
	push @snmptable, \@commrow;
}


#print "All SNMP Profiles in DA: \n";
for (my $i = 0; $i < scalar(@snmptable); $i++) {
#	print $snmptable[$i][0] . "," . $snmptable[$i][1] . "," . $snmptable[$i][2] . "\n";
}

### process DA IP domains
##################################################################################################

$client->GET('/rest/ipdomains/');
# print $client->responseContent();

my $ipdomainlist    = $parser->load_xml(string => $client->responseContent());

foreach my $ipdomain ($ipdomainlist->findnodes('/IPDomainList/IPDomain')) {
    my($title) = $ipdomain->findnodes('./Name');
    # print $title->to_literal, "\n";
	my @domrow;														
	push @domrow, $title->to_literal;
    my($title) = $ipdomain->findnodes('./ID');
    # print $title->to_literal, "\n";
	push @domrow, $title->to_literal;
	push @domaintable, \@domrow;
}
#print "All IP Domains in DA: \n";
for (my $i = 0; $i < scalar(@domaintable); $i++) {
#	print $domaintable[$i][0] . "," . $domaintable[$i][1] . "\n";
}

### process discovery profiles
##################################################################################################

# get discovery profile list by filter 
my @discprofiletable;									# profile name/id table
$client->GET('/rest/tenant/1/discoveryprofiles/');
print EXPORT $client->responseContent();

my $discprofilelist = $parser->load_xml(string => $client->responseContent());

my $discprofname;
my $discprofid;
my $snmpprofid;
my $snmpprofname;
my $snmpv;
my $domainid;
my $domainname;

print "Exporting ...\n";
print "DPName, ID, Name, ID, Type\n";
foreach my $discprofile ($discprofilelist->findnodes('/DiscoveryProfileList/DiscoveryProfile')) {
    my($title) = $discprofile->findnodes('./Item/Name');
	$discprofname = $title->to_literal;
    my($title) = $discprofile->findnodes('./ID');
	$discprofid = $title->to_literal;
	
	my $results = $discprofile->findnodes('./SNMPProfileIDList/SNMPProfileID');
	foreach my $snmpprofile ($results->get_nodelist) {
		my @discprofilerow;
		push @discprofilerow, $discprofname;
		push @discprofilerow, $discprofid;
		my($title) = $snmpprofile->findnodes('.');
		$snmpprofid = $title->to_literal;
		# print $discprofname . " snmpprofid " . $snmpprofid . "\n";

		$snmpprofname = "";
		# SNMP table lookup profile id and get SNMP profile name
		for (my $i = 0; $i < scalar(@snmptable); $i++) {
			if ($snmptable[$i][1] eq $snmpprofid) {
				$snmpprofname = $snmptable[$i][0];
				$snmpv = $snmptable[$i][2];
			}
		}
		push @discprofilerow, $snmpprofname;
		push @discprofilerow, $snmpprofid;
		push @discprofilerow, $snmpv;
		push @discprofiletable, \@discprofilerow;
	}
    my($title) = $discprofile->findnodes('./IPDomainMember/IPDomainID');
	$domainid = $title->to_literal;
	# domain table lookup profile id and get domain name
	for (my $i = 0; $i < scalar(@domaintable); $i++) {
		if ($domaintable[$i][1] eq $domainid) {
			$domainname = $domaintable[$i][0];
		}
	}
	my @discprofilerow;
	push @discprofilerow, $discprofname;
	push @discprofilerow, $discprofid;
	push @discprofilerow, $domainname;
	push @discprofilerow, $domainid;
	push @discprofilerow, "DOMAIN";
	push @discprofiletable, \@discprofilerow;
}
#print "Print discprofiletable: \n";
for (my $i = 0; $i < scalar(@discprofiletable); $i++) {
	print $discprofiletable[$i][0] . "," . $discprofiletable[$i][1] . "," . $discprofiletable[$i][2] . "," . $discprofiletable[$i][3] . "," . $discprofiletable[$i][4] . "\n";
	print ASSOC $discprofiletable[$i][0] . "," . $discprofiletable[$i][1] . "," . $discprofiletable[$i][2] . "," . $discprofiletable[$i][3] . "," . $discprofiletable[$i][4] . "\n";

}

close (EXPORT);
close (ASSOC);

 